@@ -1,5 +1,5 @@
 /*
- * Copyright 2003-2017 The Music Player Daemon Project
+ * Copyright 2003-2018 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -29,29 +29,6 @@
 
 #include <assert.h>
 
-#define GLIBCXX_490 20140422
-#define GLIBCXX_491 20140716
-#define GLIBCXX_492 20141030
-#define GLIBCXX_492_Debian_9 20141220
-#define GLIBCXX_493 20150626
-#define GLIBCXX_494 20160803
-#define GLIBCXX_49X_NDK_r13b 20150123
-
-/* the big mess attempts to detect whether we're compiling with
-   libstdc++ 4.9.x; __GLIBCXX__ is a date tag and cannot be used to
-   check the major version; and just checking the compiler version
-   isn't enough, because somebody could use an old libstdc++ with
-   clang - SIGH! */
-#if GCC_OLDER_THAN(5,0) || (defined(__GLIBCXX__) &&		     \
-	(__GLIBCXX__ == GLIBCXX_490 || __GLIBCXX__ == GLIBCXX_491 || \
-	 __GLIBCXX__ == GLIBCXX_492 || \
-	 __GLIBCXX__ == GLIBCXX_492_Debian_9 || \
-	 __GLIBCXX__ == GLIBCXX_493 || \
-	 __GLIBCXX__ == GLIBCXX_494 || \
-	 __GLIBCXX__ == GLIBCXX_49X_NDK_r13b))
-#define GLIBCXX_49X
-#endif
-
 gcc_const
 static enum ack
 ToAck(PlaylistResult result) noexcept
@@ -123,19 +100,13 @@ ToAck(std::exception_ptr ep) noexcept
 		return ACK_ERROR_SYSTEM;
 	} catch (const std::invalid_argument &e) {
 		return ACK_ERROR_ARG;
-#ifdef GLIBCXX_49X
-	} catch (const std::exception &e) {
-#else
+	} catch (const std::length_error &e) {
+		return ACK_ERROR_ARG;
+	} catch (const std::out_of_range &e) {
+		return ACK_ERROR_ARG;
 	} catch (...) {
-#endif
 		try {
-#ifdef GLIBCXX_49X
-			/* workaround for g++ 4.x: no overload for
-			   rethrow_exception(exception_ptr) */
-			std::rethrow_if_nested(e);
-#else
 			std::rethrow_if_nested(ep);
-#endif
 			return ACK_ERROR_UNKNOWN;
 		} catch (...) {
 			return ToAck(std::current_exception());
@@ -147,5 +118,5 @@ void
 PrintError(Response &r, std::exception_ptr ep)
 {
 	LogError(ep);
-	r.Error(ToAck(ep), FullMessage(ep).c_str());
+	r.Error(ToAck(ep), GetFullMessage(ep).c_str());
 }
