@@ -1,5 +1,5 @@
 /*
- * Copyright 2003-2017 The Music Player Daemon Project
+ * Copyright 2003-2018 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -37,6 +37,7 @@
 #include "fs/AllocatedPath.hxx"
 #include "util/UriUtil.hxx"
 #include "util/ConstBuffer.hxx"
+#include "util/ChronoUtil.hxx"
 
 bool
 playlist_commands_available() noexcept
@@ -50,7 +51,7 @@ print_spl_list(Response &r, const PlaylistVector &list)
 	for (const auto &i : list) {
 		r.Format("playlist: %s\n", i.name.c_str());
 
-		if (i.mtime > 0)
+		if (!IsNegative(i.mtime))
 			time_print(r, "Last-Modified", i.mtime);
 	}
 }
@@ -58,7 +59,7 @@ print_spl_list(Response &r, const PlaylistVector &list)
 CommandResult
 handle_save(Client &client, Request args, gcc_unused Response &r)
 {
-	spl_save_playlist(args.front(), client.playlist);
+	spl_save_playlist(args.front(), client.GetPlaylist());
 	return CommandResult::OK;
 }
 
@@ -67,13 +68,13 @@ handle_load(Client &client, Request args, gcc_unused Response &r)
 {
 	RangeArg range = args.ParseOptional(1, RangeArg::All());
 
-	const ScopeBulkEdit bulk_edit(client.partition);
+	const ScopeBulkEdit bulk_edit(client.GetPartition());
 
 	const SongLoader loader(client);
 	playlist_open_into_queue(args.front(),
 				 range.start, range.end,
-				 client.playlist,
-				 client.player_control, loader);
+				 client.GetPlaylist(),
+				 client.GetPlayerControl(), loader);
 	return CommandResult::OK;
 }
 
@@ -82,8 +83,8 @@ handle_listplaylist(Client &client, Request args, Response &r)
 {
 	const char *const name = args.front();
 
-	if (playlist_file_print(r, client.partition, SongLoader(client),
-				 name, false))
+	if (playlist_file_print(r, client.GetPartition(), SongLoader(client),
+				name, false))
 		return CommandResult::OK;
 
 	throw PlaylistError::NoSuchList();
@@ -94,7 +95,7 @@ handle_listplaylistinfo(Client &client, Request args, Response &r)
 {
 	const char *const name = args.front();
 
-	if (playlist_file_print(r, client.partition, SongLoader(client),
+	if (playlist_file_print(r, client.GetPartition(), SongLoader(client),
 				name, true))
 		return CommandResult::OK;
 
@@ -166,7 +167,7 @@ handle_playlistadd(Client &client, Request args, gcc_unused Response &r)
 #ifdef ENABLE_DATABASE
 		const Database &db = client.GetDatabaseOrThrow();
 
-		search_add_to_playlist(db, *client.GetStorage(),
+		search_add_to_playlist(db, client.GetStorage(),
 				       uri, playlist, nullptr);
 #else
 		r.Error(ACK_ERROR_NO_EXIST, "directory or file not found");
