@@ -1,5 +1,5 @@
 /*
- * Copyright 2003-2017 The Music Player Daemon Project
+ * Copyright 2003-2018 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -19,7 +19,7 @@
 
 #include "Object.hxx"
 
-UPnPDirObject::~UPnPDirObject()
+UPnPDirObject::~UPnPDirObject() noexcept
 {
 	/* this destructor exists here just so it won't get inlined */
 }
