@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2014 The Music Player Daemon Project
+ * Copyright 2003-2018 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -21,25 +21,27 @@
 #include "MadDecoderPlugin.hxx"
 #include "../DecoderAPI.hxx"
 #include "input/InputStream.hxx"
-#include "config/ConfigGlobal.hxx"
-#include "tag/TagId3.hxx"
-#include "tag/TagRva2.hxx"
-#include "tag/TagHandler.hxx"
+#include "config/Block.hxx"
+#include "tag/Id3Scan.hxx"
+#include "tag/Id3ReplayGain.hxx"
+#include "tag/Rva2.hxx"
+#include "tag/Handler.hxx"
 #include "tag/ReplayGain.hxx"
 #include "tag/MixRamp.hxx"
 #include "CheckAudioFormat.hxx"
-#include "util/StringUtil.hxx"
-#include "util/ASCII.hxx"
-#include "util/Error.hxx"
+#include "util/StringCompare.hxx"
 #include "util/Domain.hxx"
 #include "Log.hxx"
 
 #include <mad.h>
 
-#ifdef HAVE_ID3TAG
+#ifdef ENABLE_ID3TAG
+#include "tag/Id3Unique.hxx"
 #include <id3tag.h>
 #endif
 
+#include <stdexcept>
+
 #include <assert.h>
 #include <stdlib.h>
 #include <stdio.h>
@@ -71,7 +73,7 @@ static bool gapless_playback;
 
 gcc_const
 static SongTime
-ToSongTime(mad_timer_t t)
+ToSongTime(mad_timer_t t) noexcept
 {
 	return SongTime::FromMS(mad_timer_count(t, MAD_UNITS_MILLISECONDS));
 }
@@ -107,10 +109,10 @@ mad_fixed_to_24_buffer(int32_t *dest, const struct mad_synth *synth,
 }
 
 static bool
-mp3_plugin_init(gcc_unused const config_param &param)
+mp3_plugin_init(const ConfigBlock &block)
 {
-	gapless_playback = config_get_bool(CONF_GAPLESS_MP3_PLAYBACK,
-					   DEFAULT_GAPLESS_MP3_PLAYBACK);
+	gapless_playback = block.GetBlockValue("gapless",
+					       DEFAULT_GAPLESS_MP3_PLAYBACK);
 	return true;
 }
 
@@ -127,77 +129,78 @@ struct MadDecoder {
 	SignedSongTime total_time;
 	SongTime elapsed_time;
 	SongTime seek_time;
-	enum muteframe mute_frame;
-	long *frame_offsets;
-	mad_timer_t *times;
-	unsigned long highest_frame;
-	unsigned long max_frames;
-	unsigned long current_frame;
-	unsigned int drop_start_frames;
-	unsigned int drop_end_frames;
-	unsigned int drop_start_samples;
-	unsigned int drop_end_samples;
-	bool found_replay_gain;
-	bool found_first_frame;
-	bool decoded_first_frame;
+	enum muteframe mute_frame = MUTEFRAME_NONE;
+	long *frame_offsets = nullptr;
+	mad_timer_t *times = nullptr;
+	unsigned long highest_frame = 0;
+	unsigned long max_frames = 0;
+	unsigned long current_frame = 0;
+	unsigned int drop_start_frames = 0;
+	unsigned int drop_end_frames = 0;
+	unsigned int drop_start_samples = 0;
+	unsigned int drop_end_samples = 0;
+	bool found_replay_gain = false;
+	bool found_first_frame = false;
+	bool decoded_first_frame = false;
 	unsigned long bit_rate;
-	Decoder *const decoder;
+	DecoderClient *const client;
 	InputStream &input_stream;
-	enum mad_layer layer;
+	enum mad_layer layer = mad_layer(0);
 
-	MadDecoder(Decoder *decoder, InputStream &input_stream);
+	MadDecoder(DecoderClient *client, InputStream &input_stream);
 	~MadDecoder();
 
 	bool Seek(long offset);
 	bool FillBuffer();
-	void ParseId3(size_t tagsize, Tag **mpd_tag);
-	enum mp3_action DecodeNextFrameHeader(Tag **tag);
+	void ParseId3(size_t tagsize, Tag *tag);
+	enum mp3_action DecodeNextFrameHeader(Tag *tag);
 	enum mp3_action DecodeNextFrame();
 
 	gcc_pure
-	offset_type ThisFrameOffset() const;
+	offset_type ThisFrameOffset() const noexcept;
 
 	gcc_pure
-	offset_type RestIncludingThisFrame() const;
+	offset_type RestIncludingThisFrame() const noexcept;
 
 	/**
 	 * Attempt to calulcate the length of the song from filesize
 	 */
 	void FileSizeToSongLength();
 
-	bool DecodeFirstFrame(Tag **tag);
+	bool DecodeFirstFrame(Tag *tag);
+
+	void AllocateBuffers() {
+		assert(max_frames > 0);
+		assert(frame_offsets == nullptr);
+		assert(times == nullptr);
+
+		frame_offsets = new long[max_frames];
+		times = new mad_timer_t[max_frames];
+	}
 
 	gcc_pure
-	long TimeToFrame(SongTime t) const;
+	long TimeToFrame(SongTime t) const noexcept;
 
 	void UpdateTimerNextFrame();
 
 	/**
-	 * Sends the synthesized current frame via decoder_data().
+	 * Sends the synthesized current frame via
+	 * DecoderClient::SubmitData().
 	 */
 	DecoderCommand SendPCM(unsigned i, unsigned pcm_length);
 
 	/**
 	 * Synthesize the current frame and send it via
-	 * decoder_data().
+	 * DecoderClient::SubmitData().
 	 */
 	DecoderCommand SyncAndSend();
 
 	bool Read();
 };
 
-MadDecoder::MadDecoder(Decoder *_decoder,
+MadDecoder::MadDecoder(DecoderClient *_client,
 		       InputStream &_input_stream)
-	:mute_frame(MUTEFRAME_NONE),
-	 frame_offsets(nullptr),
-	 times(nullptr),
-	 highest_frame(0), max_frames(0), current_frame(0),
-	 drop_start_frames(0), drop_end_frames(0),
-	 drop_start_samples(0), drop_end_samples(0),
-	 found_replay_gain(false),
-	 found_first_frame(false), decoded_first_frame(false),
-	 decoder(_decoder), input_stream(_input_stream),
-	 layer(mad_layer(0))
+	:client(_client), input_stream(_input_stream)
 {
 	mad_stream_init(&stream);
 	mad_stream_options(&stream, MAD_OPTION_IGNORECRC);
@@ -209,9 +212,11 @@ MadDecoder::MadDecoder(Decoder *_decoder,
 inline bool
 MadDecoder::Seek(long offset)
 {
-	Error error;
-	if (!input_stream.LockSeek(offset, error))
+	try {
+		input_stream.LockSeek(offset);
+	} catch (...) {
 		return false;
+	}
 
 	mad_stream_buffer(&stream, input_buffer, 0);
 	stream.error = MAD_ERROR_NONE;
@@ -241,7 +246,7 @@ MadDecoder::FillBuffer()
 	if (length == 0)
 		return false;
 
-	length = decoder_read(decoder, input_stream, dest, length);
+	length = decoder_read(client, input_stream, dest, length);
 	if (length == 0)
 		return false;
 
@@ -251,44 +256,10 @@ MadDecoder::FillBuffer()
 	return true;
 }
 
-#ifdef HAVE_ID3TAG
-static bool
-parse_id3_replay_gain_info(ReplayGainInfo &rgi,
-			   struct id3_tag *tag)
-{
-	bool found = false;
-
-	rgi.Clear();
-
-	struct id3_frame *frame;
-	for (unsigned i = 0; (frame = id3_tag_findframe(tag, "TXXX", i)); i++) {
-		if (frame->nfields < 3)
-			continue;
-
-		char *const key = (char *)
-		    id3_ucs4_latin1duplicate(id3_field_getstring
-					     (&frame->fields[1]));
-		char *const value = (char *)
-		    id3_ucs4_latin1duplicate(id3_field_getstring
-					     (&frame->fields[2]));
-
-		if (ParseReplayGainTag(rgi, key, value))
-			found = true;
-
-		free(key);
-		free(value);
-	}
-
-	return found ||
-		/* fall back on RVA2 if no replaygain tags found */
-		tag_rva2_parse(tag, rgi);
-}
-#endif
-
-#ifdef HAVE_ID3TAG
+#ifdef ENABLE_ID3TAG
 gcc_pure
 static MixRampInfo
-parse_id3_mixramp(struct id3_tag *tag)
+parse_id3_mixramp(struct id3_tag *tag) noexcept
 {
 	MixRampInfo result;
 
@@ -315,10 +286,10 @@ parse_id3_mixramp(struct id3_tag *tag)
 #endif
 
 inline void
-MadDecoder::ParseId3(size_t tagsize, Tag **mpd_tag)
+MadDecoder::ParseId3(size_t tagsize, Tag *mpd_tag)
 {
-#ifdef HAVE_ID3TAG
-	id3_byte_t *allocated = nullptr;
+#ifdef ENABLE_ID3TAG
+	std::unique_ptr<id3_byte_t[]> allocated;
 
 	const id3_length_t count = stream.bufend - stream.this_frame;
 
@@ -327,49 +298,38 @@ MadDecoder::ParseId3(size_t tagsize, Tag **mpd_tag)
 		id3_data = stream.this_frame;
 		mad_stream_skip(&(stream), tagsize);
 	} else {
-		allocated = new id3_byte_t[tagsize];
-		memcpy(allocated, stream.this_frame, count);
+		allocated.reset(new id3_byte_t[tagsize]);
+		memcpy(allocated.get(), stream.this_frame, count);
 		mad_stream_skip(&(stream), count);
 
-		if (!decoder_read_full(decoder, input_stream,
-				       allocated + count, tagsize - count)) {
+		if (!decoder_read_full(client, input_stream,
+				       allocated.get() + count, tagsize - count)) {
 			LogDebug(mad_domain, "error parsing ID3 tag");
-			delete[] allocated;
 			return;
 		}
 
-		id3_data = allocated;
+		id3_data = allocated.get();
 	}
 
-	struct id3_tag *const id3_tag = id3_tag_parse(id3_data, tagsize);
-	if (id3_tag == nullptr) {
-		delete[] allocated;
+	const UniqueId3Tag id3_tag(id3_tag_parse(id3_data, tagsize));
+	if (id3_tag == nullptr)
 		return;
-	}
 
-	if (mpd_tag) {
-		Tag *tmp_tag = tag_id3_import(id3_tag);
-		if (tmp_tag != nullptr) {
-			delete *mpd_tag;
-			*mpd_tag = tmp_tag;
-		}
-	}
+	if (mpd_tag != nullptr)
+		*mpd_tag = tag_id3_import(id3_tag.get());
 
-	if (decoder != nullptr) {
+	if (client != nullptr) {
 		ReplayGainInfo rgi;
 
-		if (parse_id3_replay_gain_info(rgi, id3_tag)) {
-			decoder_replay_gain(*decoder, &rgi);
+		if (Id3ToReplayGainInfo(rgi, id3_tag.get())) {
+			client->SubmitReplayGain(&rgi);
 			found_replay_gain = true;
 		}
 
-		decoder_mixramp(*decoder, parse_id3_mixramp(id3_tag));
+		client->SubmitMixRamp(parse_id3_mixramp(id3_tag.get()));
 	}
 
-	id3_tag_delete(id3_tag);
-
-	delete[] allocated;
-#else /* !HAVE_ID3TAG */
+#else /* !ENABLE_ID3TAG */
 	(void)mpd_tag;
 
 	/* This code is enabled when libid3tag is disabled.  Instead
@@ -381,12 +341,12 @@ MadDecoder::ParseId3(size_t tagsize, Tag **mpd_tag)
 		mad_stream_skip(&stream, tagsize);
 	} else {
 		mad_stream_skip(&stream, count);
-		decoder_skip(decoder, input_stream, tagsize - count);
+		decoder_skip(client, input_stream, tagsize - count);
 	}
 #endif
 }
 
-#ifndef HAVE_ID3TAG
+#ifndef ENABLE_ID3TAG
 /**
  * This function emulates libid3tag when it is disabled.  Instead of
  * doing a real analyzation of the frame, it just checks whether the
@@ -402,7 +362,7 @@ id3_tag_query(const void *p0, size_t length)
 		? (p[8] << 7) + p[9] + 10
 		: 0;
 }
-#endif /* !HAVE_ID3TAG */
+#endif /* !ENABLE_ID3TAG */
 
 static enum mp3_action
 RecoverFrameError(struct mad_stream &stream)
@@ -419,7 +379,7 @@ RecoverFrameError(struct mad_stream &stream)
 }
 
 enum mp3_action
-MadDecoder::DecodeNextFrameHeader(Tag **tag)
+MadDecoder::DecodeNextFrameHeader(Tag *tag)
 {
 	if ((stream.buffer == nullptr || stream.error == MAD_ERROR_BUFLEN) &&
 	    !FillBuffer())
@@ -432,11 +392,7 @@ MadDecoder::DecodeNextFrameHeader(Tag **tag)
 							    stream.this_frame);
 
 			if (tagsize > 0) {
-				if (tag && !(*tag)) {
-					ParseId3((size_t)tagsize, tag);
-				} else {
-					mad_stream_skip(&stream, tagsize);
-				}
+				ParseId3((size_t)tagsize, tag);
 				return DECODE_CONT;
 			}
 		}
@@ -490,24 +446,18 @@ static constexpr unsigned NG_MAGIC = (('n' << 8) | 'g');
 static constexpr unsigned IN_MAGIC = (('I' << 8) | 'n');
 static constexpr unsigned FO_MAGIC = (('f' << 8) | 'o');
 
-enum xing_magic {
-	XING_MAGIC_XING, /* VBR */
-	XING_MAGIC_INFO  /* CBR */
-};
-
 struct xing {
 	long flags;             /* valid fields (see below) */
 	unsigned long frames;   /* total number of frames */
 	unsigned long bytes;    /* total number of bytes */
 	unsigned char toc[100]; /* 100-point seek table */
 	long scale;             /* VBR quality */
-	enum xing_magic magic;  /* header magic */
 };
 
-static const unsigned XING_FRAMES = 1;
-static const unsigned XING_BYTES = 2;
-static const unsigned XING_TOC = 4;
-static const unsigned XING_SCALE = 8;
+static constexpr unsigned XING_FRAMES = 1;
+static constexpr unsigned XING_BYTES = 2;
+static constexpr unsigned XING_TOC = 4;
+static constexpr unsigned XING_SCALE = 8;
 
 struct lame_version {
 	unsigned major;
@@ -544,7 +494,6 @@ parse_xing(struct xing *xing, struct mad_bitptr *ptr, int *oldbitlen)
 			return false;
 
 		bitlen -= 16;
-		xing->magic = XING_MAGIC_XING;
 	} else if (bits == IN_MAGIC) {
 		if (bitlen < 16)
 			return false;
@@ -553,11 +502,8 @@ parse_xing(struct xing *xing, struct mad_bitptr *ptr, int *oldbitlen)
 			return false;
 
 		bitlen -= 16;
-		xing->magic = XING_MAGIC_INFO;
 	}
-	else if (bits == NG_MAGIC) xing->magic = XING_MAGIC_XING;
-	else if (bits == FO_MAGIC) xing->magic = XING_MAGIC_INFO;
-	else
+	else if (bits != NG_MAGIC && bits != FO_MAGIC)
 		return false;
 
 	if (bitlen < 32)
@@ -600,7 +546,7 @@ parse_xing(struct xing *xing, struct mad_bitptr *ptr, int *oldbitlen)
 	if (bitsleft < 0)
 		return false;
 	else if (bitsleft > 0) {
-		mad_bit_read(ptr, bitsleft);
+		mad_bit_skip(ptr, bitsleft);
 		bitlen -= bitsleft;
 	}
 
@@ -648,7 +594,7 @@ parse_lame(struct lame *lame, struct mad_bitptr *ptr, int *bitlen)
 	    (lame->version.major == 3 && lame->version.minor < 95))
 		adj = 6;
 
-	mad_bit_read(ptr, 16);
+	mad_bit_skip(ptr, 16);
 
 	lame->peak = mad_f_todouble(mad_bit_read(ptr, 32) << 5); /* peak */
 	FormatDebug(mad_domain, "LAME peak found: %f", lame->peak);
@@ -680,10 +626,10 @@ parse_lame(struct lame *lame, struct mad_bitptr *ptr, int *bitlen)
 			    lame->track_gain);
 	}
 #else
-	mad_bit_read(ptr, 16);
+	mad_bit_skip(ptr, 16);
 #endif
 
-	mad_bit_read(ptr, 16);
+	mad_bit_skip(ptr, 16);
 
 	lame->encoder_delay = mad_bit_read(ptr, 12);
 	lame->encoder_padding = mad_bit_read(ptr, 12);
@@ -691,7 +637,7 @@ parse_lame(struct lame *lame, struct mad_bitptr *ptr, int *bitlen)
 	FormatDebug(mad_domain, "encoder delay is %i, encoder padding is %i",
 		    lame->encoder_delay, lame->encoder_padding);
 
-	mad_bit_read(ptr, 80);
+	mad_bit_skip(ptr, 80);
 
 	lame->crc = mad_bit_read(ptr, 16);
 
@@ -707,7 +653,7 @@ mp3_frame_duration(const struct mad_frame *frame)
 }
 
 inline offset_type
-MadDecoder::ThisFrameOffset() const
+MadDecoder::ThisFrameOffset() const noexcept
 {
 	auto offset = input_stream.GetOffset();
 
@@ -720,7 +666,7 @@ MadDecoder::ThisFrameOffset() const
 }
 
 inline offset_type
-MadDecoder::RestIncludingThisFrame() const
+MadDecoder::RestIncludingThisFrame() const noexcept
 {
 	return input_stream.GetSize() - ThisFrameOffset();
 }
@@ -748,10 +694,9 @@ MadDecoder::FileSizeToSongLength()
 }
 
 inline bool
-MadDecoder::DecodeFirstFrame(Tag **tag)
+MadDecoder::DecodeFirstFrame(Tag *tag)
 {
 	struct xing xing;
-	xing.frames = 0;
 
 	while (true) {
 		enum mp3_action ret;
@@ -798,13 +743,13 @@ MadDecoder::DecodeFirstFrame(Tag **tag)
 
 			/* Album gain isn't currently used.  See comment in
 			 * parse_lame() for details. -- jat */
-			if (decoder != nullptr && !found_replay_gain &&
+			if (client != nullptr && !found_replay_gain &&
 			    lame.track_gain) {
 				ReplayGainInfo rgi;
 				rgi.Clear();
-				rgi.tuples[REPLAY_GAIN_TRACK].gain = lame.track_gain;
-				rgi.tuples[REPLAY_GAIN_TRACK].peak = lame.peak;
-				decoder_replay_gain(*decoder, &rgi);
+				rgi.track.gain = lame.track_gain;
+				rgi.track.peak = lame.peak;
+				client->SubmitReplayGain(&rgi);
 			}
 		}
 	}
@@ -819,9 +764,6 @@ MadDecoder::DecodeFirstFrame(Tag **tag)
 		return false;
 	}
 
-	frame_offsets = new long[max_frames];
-	times = new mad_timer_t[max_frames];
-
 	return true;
 }
 
@@ -835,18 +777,8 @@ MadDecoder::~MadDecoder()
 	delete[] times;
 }
 
-/* this is primarily used for getting total time for tags */
-static std::pair<bool, SignedSongTime>
-mad_decoder_total_file_time(InputStream &is)
-{
-	MadDecoder data(nullptr, is);
-	return data.DecodeFirstFrame(nullptr)
-		? std::make_pair(true, data.total_time)
-		: std::make_pair(false, SignedSongTime::Negative());
-}
-
 long
-MadDecoder::TimeToFrame(SongTime t) const
+MadDecoder::TimeToFrame(SongTime t) const noexcept
 {
 	unsigned long i;
 
@@ -904,9 +836,9 @@ MadDecoder::SendPCM(unsigned i, unsigned pcm_length)
 				       MAD_NCHANNELS(&frame.header));
 		num_samples *= MAD_NCHANNELS(&frame.header);
 
-		auto cmd = decoder_data(*decoder, input_stream, output_buffer,
-					sizeof(output_buffer[0]) * num_samples,
-					bit_rate / 1000);
+		auto cmd = client->SubmitData(input_stream, output_buffer,
+					      sizeof(output_buffer[0]) * num_samples,
+					      bit_rate / 1000);
 		if (cmd != DecoderCommand::NONE)
 			return cmd;
 	}
@@ -986,18 +918,18 @@ MadDecoder::Read()
 		if (cmd == DecoderCommand::SEEK) {
 			assert(input_stream.IsSeekable());
 
-			unsigned long j =
-				TimeToFrame(decoder_seek_time(*decoder));
+			const auto t = client->GetSeekTime();
+			unsigned long j = TimeToFrame(t);
 			if (j < highest_frame) {
 				if (Seek(frame_offsets[j])) {
 					current_frame = j;
-					decoder_command_finished(*decoder);
+					client->CommandFinished();
 				} else
-					decoder_seek_error(*decoder);
+					client->SeekError();
 			} else {
-				seek_time = decoder_seek_time(*decoder);
+				seek_time = t;
 				mute_frame = MUTEFRAME_SEEK;
-				decoder_command_finished(*decoder);
+				client->CommandFinished();
 			}
 		} else if (cmd != DecoderCommand::NONE)
 			return false;
@@ -1006,15 +938,13 @@ MadDecoder::Read()
 	while (true) {
 		enum mp3_action ret;
 		do {
-			Tag *tag = nullptr;
+			Tag tag;
 
 			ret = DecodeNextFrameHeader(&tag);
 
-			if (tag != nullptr) {
-				decoder_tag(*decoder, input_stream,
-					    std::move(*tag));
-				delete tag;
-			}
+			if (!tag.IsEmpty())
+				client->SubmitTag(input_stream,
+						  std::move(tag));
 		} while (ret == DECODE_CONT);
 		if (ret == DECODE_BREAK)
 			return false;
@@ -1035,55 +965,49 @@ MadDecoder::Read()
 }
 
 static void
-mp3_decode(Decoder &decoder, InputStream &input_stream)
+mp3_decode(DecoderClient &client, InputStream &input_stream)
 {
-	MadDecoder data(&decoder, input_stream);
+	MadDecoder data(&client, input_stream);
 
-	Tag *tag = nullptr;
+	Tag tag;
 	if (!data.DecodeFirstFrame(&tag)) {
-		delete tag;
-
-		if (decoder_get_command(decoder) == DecoderCommand::NONE)
+		if (client.GetCommand() == DecoderCommand::NONE)
 			LogError(mad_domain,
 				 "input/Input does not appear to be a mp3 bit stream");
 		return;
 	}
 
-	Error error;
-	AudioFormat audio_format;
-	if (!audio_format_init_checked(audio_format,
-				       data.frame.header.samplerate,
-				       SampleFormat::S24_P32,
-				       MAD_NCHANNELS(&data.frame.header),
-				       error)) {
-		LogError(error);
-		delete tag;
-		return;
-	}
+	data.AllocateBuffers();
 
-	decoder_initialized(decoder, audio_format,
-			    input_stream.IsSeekable(),
-			    data.total_time);
+	client.Ready(CheckAudioFormat(data.frame.header.samplerate,
+				      SampleFormat::S24_P32,
+				      MAD_NCHANNELS(&data.frame.header)),
+		     input_stream.IsSeekable(),
+		     data.total_time);
 
-	if (tag != nullptr) {
-		decoder_tag(decoder, input_stream, std::move(*tag));
-		delete tag;
-	}
+	if (!tag.IsEmpty())
+		client.SubmitTag(input_stream, std::move(tag));
 
 	while (data.Read()) {}
 }
 
 static bool
-mad_decoder_scan_stream(InputStream &is,
-			const struct tag_handler *handler, void *handler_ctx)
+mad_decoder_scan_stream(InputStream &is, TagHandler &handler) noexcept
 {
-	const auto result = mad_decoder_total_file_time(is);
-	if (!result.first)
+	MadDecoder data(nullptr, is);
+	if (!data.DecodeFirstFrame(nullptr))
 		return false;
 
-	if (!result.second.IsNegative())
-		tag_handler_invoke_duration(handler, handler_ctx,
-					    SongTime(result.second));
+	if (!data.total_time.IsNegative())
+		handler.OnDuration(SongTime(data.total_time));
+
+	try {
+		handler.OnAudioFormat(CheckAudioFormat(data.frame.header.samplerate,
+						       SampleFormat::S24_P32,
+						       MAD_NCHANNELS(&data.frame.header)));
+	} catch (...) {
+	}
+
 	return true;
 }
 
