@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2010-2011 Max Kellermann <max@duempel.org>
+ * Copyright 2010-2018 Max Kellermann <max.kellermann@gmail.com>
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -32,7 +32,7 @@
 namespace Java {
 	JavaVM *jvm;
 
-	void Init(JNIEnv *env)
+	void Init(JNIEnv *env) noexcept
 	{
 		env->GetJavaVM(&jvm);
 	}
