@@ -1,5 +1,5 @@
 /*
- * Copyright 2003-2017 The Music Player Daemon Project
+ * Copyright 2003-2018 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -24,13 +24,14 @@
 #include <string>
 
 static std::string
-MakeSqliteErrorMessage(sqlite3 *db, const char *msg)
+MakeSqliteErrorMessage(sqlite3 *db, const char *msg) noexcept
 {
 	return std::string(msg) + ": " + sqlite3_errmsg(db);
 }
 
-SqliteError::SqliteError(sqlite3 *db, int _code, const char *msg)
+SqliteError::SqliteError(sqlite3 *db, int _code, const char *msg) noexcept
 	:std::runtime_error(MakeSqliteErrorMessage(db, msg)), code(_code) {}
 
-SqliteError::SqliteError(sqlite3_stmt *stmt, int _code, const char *msg)
+SqliteError::SqliteError(sqlite3_stmt *stmt, int _code,
+			 const char *msg) noexcept
 	:SqliteError(sqlite3_db_handle(stmt), _code, msg) {}
