@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2014 The Music Player Daemon Project
+ * Copyright 2003-2018 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -18,13 +18,14 @@
  */
 
 #include "config.h"
-#include "tag/TagId3.hxx"
-#include "tag/TagRva2.hxx"
+#include "tag/Id3Load.hxx"
+#include "tag/Rva2.hxx"
 #include "ReplayGainInfo.hxx"
-#include "config/ConfigGlobal.hxx"
-#include "util/Error.hxx"
+#include "thread/Mutex.hxx"
 #include "fs/Path.hxx"
-#include "Log.hxx"
+#include "input/InputStream.hxx"
+#include "input/LocalOpen.hxx"
+#include "util/PrintException.hxx"
 
 #include <id3tag.h>
 
@@ -35,15 +36,23 @@
 #include <stdlib.h>
 #include <stdio.h>
 
-const char *
-config_get_string(gcc_unused enum ConfigOption option,
-		  const char *default_value)
+static void
+DumpReplayGainTuple(const char *name, const ReplayGainTuple &tuple)
 {
-	return default_value;
+	if (tuple.IsDefined())
+		fprintf(stderr, "replay_gain[%s]: gain=%f peak=%f\n",
+			name, tuple.gain, tuple.peak);
+}
+
+static void
+DumpReplayGainInfo(const ReplayGainInfo &info)
+{
+	DumpReplayGainTuple("album", info.album);
+	DumpReplayGainTuple("track", info.track);
 }
 
 int main(int argc, char **argv)
-{
+try {
 #ifdef HAVE_LOCALE_H
 	/* initialize locale */
 	setlocale(LC_CTYPE,"");
@@ -54,39 +63,31 @@ int main(int argc, char **argv)
 		return EXIT_FAILURE;
 	}
 
-	const char *path = argv[1];
+	const Path path = Path::FromFS(argv[1]);
 
-	Error error;
-	struct id3_tag *tag = tag_id3_load(Path::FromFS(path), error);
+	Mutex mutex;
+
+	auto is = OpenLocalInputStream(path, mutex);
+
+	const auto tag = tag_id3_load(*is);
 	if (tag == NULL) {
-		if (error.IsDefined())
-			LogError(error);
-		else
-			fprintf(stderr, "No ID3 tag found\n");
-
+		fprintf(stderr, "No ID3 tag found\n");
 		return EXIT_FAILURE;
 	}
 
 	ReplayGainInfo replay_gain;
 	replay_gain.Clear();
 
-	bool success = tag_rva2_parse(tag, replay_gain);
-	id3_tag_delete(tag);
-
+	bool success = tag_rva2_parse(tag.get(), replay_gain);
 	if (!success) {
 		fprintf(stderr, "No RVA2 tag found\n");
 		return EXIT_FAILURE;
 	}
 
-	const ReplayGainTuple *tuple = &replay_gain.tuples[REPLAY_GAIN_ALBUM];
-	if (tuple->IsDefined())
-		fprintf(stderr, "replay_gain[album]: gain=%f peak=%f\n",
-			tuple->gain, tuple->peak);
-
-	tuple = &replay_gain.tuples[REPLAY_GAIN_TRACK];
-	if (tuple->IsDefined())
-		fprintf(stderr, "replay_gain[track]: gain=%f peak=%f\n",
-			tuple->gain, tuple->peak);
+	DumpReplayGainInfo(replay_gain);
 
 	return EXIT_SUCCESS;
+} catch (...) {
+	PrintException(std::current_exception());
+	return EXIT_FAILURE;
 }
